﻿using System.Collections.Generic;
using System.Linq;

namespace Rendszerhiba_Újratöltve
{
    static class MarkdownÉrtelmező
    {
        /// <summary>
        /// Markdown formátumú szöveg értelmezése
        /// </summary>
        /// <returns>Az felépített HTMLDokumentum</returns>
        /// <param name="értelmezendőSzöveg">Markdown formátumú szöveg</param>
        public static HTMLDokumentum Értelmez(string értelmezendőSzöveg)
        {
            return new HTMLDokumentum(from bekezdés in BekezdésekreBontás(értelmezendőSzöveg)
                                      select BekezdésÉrtelemezés(bekezdés));
        }

        /// <summary>
        /// Bekezdésekre bontja a bemenetet. Az eredmény nem tartalmaz újsort.
        /// </summary>
        private static IEnumerable<string> BekezdésekreBontás(string szöveg)
        {
            szöveg = szöveg.Replace("\r\n", "\n");
            string[] sorok = szöveg.Split('\n');
            List<string> bekezdések = new List<string>();
            bool vége = true;
            foreach (string sor in sorok)
            {
                if (string.IsNullOrWhiteSpace(sor))
                {
                    vége = true;
                }
                else if (vége)
                {
                    bekezdések.Add(sor);
                    vége = false;
                }
                else
                {
                    bekezdések[bekezdések.Count - 1] += " " + sor;
                }
            }
            return bekezdések;
        }

        private static IHTMLElem BekezdésÉrtelemezés(string szöveg)
        {
            szöveg = szöveg.TrimStart();
            if (szöveg.StartsWith("#"))
            {
                return CímsorÉrtelemezés(szöveg);
            }
            else
            {
                return new HTMLBekezdés(EgyszerűSzövegÉrtelmezés(szöveg));
            }
        }

        private static IHTMLElem CímsorÉrtelemezés(string szöveg)
        {
            int index = szöveg.IndexOf(' ');
            return new HTMLCímsor(index, EgyszerűSzövegÉrtelmezés(szöveg.Substring(index + 1)));
        }

        private static IEnumerable<IMarkdownElemÉrtelmező> értelmezők = new IMarkdownElemÉrtelmező[]
            {
                new MarkdownBackslashÉrtelemző(),
                new MarkdownLinkÉrtelmező(),
                new MarkdownLinkVégÉrtelmező(),
                new MarkdownEgyszerűKarakterÉrtelmező(),
            };

        public static IEnumerable<IHTMLElem> EgyszerűSzövegÉrtelmezés(string szöveg)
        {
            int index = 0;
            return EgyszerűSzövegÉrtelmezés(szöveg, ref index);
        }

        /// <summary>
        /// //Rekurzívan értelmezhető elemek értelmezése
        /// </summary>
        public static IEnumerable<IHTMLElem> SzövegrészÉrtelmezés(string szöveg, ref int index)
        {
            List<IHTMLElem> eredmény = new List<IHTMLElem>();
            while (index != szöveg.Length)
            {
                if (szöveg[index] == '*')
                {
                    return eredmény;
                }
                
                int index2 = index; //Mert lambda kifejezésben nem használható ref változó

                IMarkdownElemÉrtelmező értelmező = értelmezők.First(é => é.Értelemezhető(szöveg, index2));
                IHTMLElem elem = értelmező.Értelmez(szöveg, ref index);
                if (elem == null)
                {
                    return eredmény;
                }
                eredmény.Add(elem);
            }
            return eredmény;
        }

        public static IEnumerable<IHTMLElem> EgyszerűSzövegÉrtelmezés(string szöveg, ref int index)
        {
            //Ezek azért kellenek, mert a nyelvet nem lehet rekurzívan leírni.
            bool fontos = false;
            bool kiemelt = false;
            List<IHTMLElem> eredmény = new List<IHTMLElem>();
            while (index != szöveg.Length)
            {
                IEnumerable<IHTMLElem> elemek = SzövegrészÉrtelmezés(szöveg, ref index);
                if (fontos)
                {
                    if (kiemelt)
                    {
                        eredmény.Add(new HTMLFontos(new[] { new HTMLKiemelés(elemek) }));
                    }
                    else
                    {
                        eredmény.Add(new HTMLFontos(elemek));
                    }
                }
                else
                {
                    if (kiemelt)
                    {
                        eredmény.Add(new HTMLKiemelés(elemek));
                    }
                    else
                    {
                        eredmény.AddRange(elemek);
                    }
                }
                if (index != szöveg.Length && szöveg[index] == '*')
                {
                    index++;
                    if (szöveg.Length >= index + 1 && szöveg[index] == '*')
                    {
                        //Két *
                        fontos = !fontos;
                        index++;
                    }
                    else
                    {
                        //Egy *
                        kiemelt = !kiemelt;
                    }

                }
            }
            return eredmény;
        }
    }
}
